@echo OFF
@if not "%ECHO%"=="" echo %ECHO%
@if not "%OS%"=="Windows_NT" goto DOSEXIT
rem $Workfile: repl.bat $ $Revision: 1 $ $Date: 12/04/97 9:51a $
rem $Archive: /TimH/Pubs/Books/Macmillan/Windows NT Scripting/Scripts/repl.bat $

rem Set local scope and call MAIN procedure
setlocal & pushd & set RET=
	set SCRIPTNAME=%~n0
	set SCRIPTPATH=%~f0
	if "%DEBUG%"=="1" (set TRACE=echo) else (set TRACE=rem)
	call _mtplib :INIT %SCRIPTPATH%
	if /i {%1}=={/help} (call :HELP %2) & (goto :HELPEXIT)
	if /i {%1}=={/?} (call :HELP %2) & (goto :HELPEXIT)
	call :MAIN %*
	:HELPEXIT
popd & endlocal & set RET=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem HELP procedure
rem Display brief on-line help message
rem
:HELP
if defined TRACE %TRACE% [proc %0 %*]
	echo Syntax: REPL [switches]
	echo Automates peer-to-peer directory and file replication.
	echo .
	echo REPL [site-name]
	echo     Display current replication status for all sites or for
	echo     specified site.
	echo REPL site-name /ADD:dir [...]
	echo     Add specified directories to replication site.
	echo REPL site-name /DELETE:dir [...]
	echo     Delete specified directories from replication site.
	echo REPL site-name /DELALL [/Y]
	echo     Delete all directories from specified site. /Y skips confirm.
	echo REPL site-name /ENABLE ^| /DISABLE
	echo     Enable or disable an individual site.
	echo REPL /RESET [/Y]
	echo     Delete all sites. /Y skips confirm.
	echo REPL /START [/INTERVAL:nnn] [/FIRST:nnn] [/LOG:log-file]
	echo     Start automatic replication to run every nnn minutes. The
	echo     default is once per hour. The first will occur in nnn minutes.
	echo REPL /STOP
	echo     Stop automatic replication.
	echo REPL /RUN [/LOG:log-file]
	echo     Execute a manual replication run.
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem MAIN procedure
rem
:MAIN
if defined TRACE %TRACE% [proc %0 %*]
	rem Parse command line and check for basic switches etc
	set CONTEXT=%SCRIPTNAME%.BAT
	set CMDLINE=%*
	call _mtplib :PARSECMDLINE 0
	call _mtplib :FINDSWITCH /y
	if "%RET%"=="0" (set NOCONFIRM=0) else (set NOCONFIRM=1)
	call _mtplib :GETARG 1
	set SITENAME=%RET%

	rem Dispatch to handler based upon arguments and switches
	if "%CMDSWCOUNT%"=="0" (call :REPLSHOW) & (goto :EOF)
	call _mtplib :FINDSWITCH /reset
	if not "%RET%"=="0" (call :REPLRESET) & (goto :EOF)
	call _mtplib :FINDSWITCH /start
	if not "%RET%"=="0" (call :REPLSTART) & (goto :EOF)
	call _mtplib :FINDSWITCH /stop
	if not "%RET%"=="0" (call :REPLSTOP) & (goto :EOF)
	call _mtplib :FINDSWITCH /run
	if not "%RET%"=="0" (call :REPLRUN) & (goto :EOF)
	if "%SITENAME%"=="" (echo repl: site name required) & (goto :EOF)
	call :REPLEDIT
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem REPLSHOW procedure
rem Show status of one or all sites
rem
rem Arguments:	SITENAME=name of site to display, empty for all sites
rem
:REPLSHOW
if defined TRACE %TRACE% [proc %0 %*]
	rem Load site information and display it
	call :LOADREG
	if not "%SITENAME%"=="" (call :REPLSHOW1 RSENABLE_%SITENAME%) & (goto :REPLSHOW9)
	for /f "tokens=1 delims==" %%I in ('set RSENABLE_ 2^>nul') do call :REPLSHOW1 %%I
	:REPLSHOW9

	rem Display run status
	call :GETREPLJOBID
	if "%RET%"=="NONE" (
		echo Replication is not running.
	) else (
		echo Replication is running.
	)
goto :EOF
:REPLSHOW1
	if not defined %1 goto :EOF
	for /f "tokens=2 delims=_" %%J in ("%1") do set T1=%%J
	call :LOADSITE %T1%
	if "%WORKENABLE%"=="1" (set RET=enabled) else (set RET=disabled)
	echo Replication site %T1% (%WORKCOUNT% dirs) is %RET%:
	for /l %%J in (1,1,%WORKCOUNT%) do call :REPLSHOW2 %%J
goto :EOF
:REPLSHOW2
	set RET=%%WORKDIR_%1_%%
	call _mtplib :RESOLVE
	echo      %RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem REPLRESET procedure
rem Reset (delete) all replication sites (requires confirmation)
rem
:REPLRESET
if defined TRACE %TRACE% [proc %0 %*]
	rem Confirm deletion is ok
	if not "%NOCONFIRM%"=="1" (
		%COMSPEC% /c choice /n "Delete all sites?"
		if errorlevel 2 goto :EOF
	)

	rem Delete all registry contents for script
	call _mtplib :REGDELM %CONTEXT%
	echo All sites deleted.
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem REPLSTART procedure
rem Start automatic replication
rem
rem Arguments:	/interval:nnn=set interval to nnn minutes (def: 60)
rem		/first:nnn=do first run in nnn minutes (def: 1)
rem		/log:file=send o/p to log-file (def: none)
rem
:REPLSTART
if defined TRACE %TRACE% [proc %0 %*]
	rem First, stop any current replication
	call :GETREPLJOBID
	if not "%RET%"=="NONE" at %RET% /delete

	rem Get logfile info
	set LOGFILE=
	set LOGCMD=
	call _mtplib :FINDSWITCH /log
	if not "%RET%"=="0" (set LOGFILE=%RETV%) & (set LOGCMD=/log:%RETV%)

	rem Get values for interval and first run delay
	set /a INTERVAL=60
	call _mtplib :FINDSWITCH /interval
	if not "%RET%"=="0" set /a INTERVAL=RETV
	set /a FIRST=1
	call _mtplib :FINDSWITCH /first
	if not "%RET%"=="0" set /a FIRST=RETV

	rem Schedule the first AT command via SOON
	set /a FIRST*=60
	soon %FIRST% "%COMSPEC% /c %SCRIPTPATH% /run /interval:%INTERVAL% %LOGCMD%" >nul
	echo Replication started.
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem REPLSTOP procedure
rem Stop automatic replication
rem
:REPLSTOP
if defined TRACE %TRACE% [proc %0 %*]
	call :GETREPLJOBID
	if "%RET%"=="NONE" (
		echo Replication is not running.
	) else (
		at %RET% /delete
		echo Replication stopped. 
	)
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem REPLRUN procedure
rem Perform a replication run (may be invoked from AT)
rem
rem Arguments:	/interval:nnn=set interval to nnn minutes (def: 60)
rem		/log:file=send o/p to log-file (def: none)
rem
:REPLRUN
if defined TRACE %TRACE% [proc %0 %*]
	rem Get logfile info
	set LOGFILE=
	set LOGCMD=
	call _mtplib :FINDSWITCH /log
	if not "%RET%"=="0" (set LOGFILE=%RETV%) & (set LOGCMD=/log:%RETV%)

	rem Regenerate next run if required
	call _mtplib :FINDSWITCH /interval
	if "%RET%"=="0" goto :NOREGEN
		set /a INTERVAL=RETV
		set /a RETV*=60
		soon %RETV% "%COMSPEC% /c %SCRIPTPATH% /run /interval:%INTERVAL% %LOGCMD%" >nul
	:NOREGEN

	rem Load all sites and replicate them
	if "%LOGFILE%"=="" (set LOGCMD=^>nul) else (set LOGCMD=^>^>%LOGFILE%)
	date /t %LOGCMD%
	time /t %LOGCMD%
	call :LOADREG
	for /f "tokens=1 delims==" %%I in ('set RSENABLE_ 2^>nul') do call :REPLRUN1 %%I
goto :EOF
:REPLRUN1
	for /f "tokens=2 delims=_" %%J in ("%1") do set T1=%%J
	call :LOADSITE %T1%
	if %WORKENABLE%==0 goto :EOF
	if %WORKCOUNT% LEQ 1 goto :EOF
	set /a SRCIX=1
	set /a DSTIX=2
	:REPLRUNLOOP
		if %DSTIX% GTR %WORKCOUNT% goto :REPLRUNLOOPEND
		call :REPLRUN2 %SRCIX% %DSTIX%
		set /a SRCIX+=1
		set /a DSTIX+=1
	goto :REPLRUNLOOP
	:REPLRUNLOOPEND
	call :REPLRUN2 %SRCIX% 1
goto :EOF
:REPLRUN2
	set RET=%%WORKDIR_%1_%%
	call _mtplib :RESOLVE
	set SRC=%RET%
	set RET=%%WORKDIR_%2_%%
	call _mtplib :RESOLVE
	set DST=%RET%
	echo --- robocopy "%SRC%" "%DST%" /e %LOGCMD%
	robocopy "%SRC%" "%DST%" /e %LOGCMD%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem REPLEDIT procedure
rem Edit a site contents
rem
rem Arguments:	/add:dir=add specified directory to site
rem		/delete:dir=delete specified directory from site
rem		/delall=delete entire site (requires confirm)
rem		/enable=enable site
rem		/disable=disable site
rem
:REPLEDIT
if defined TRACE %TRACE% [proc %0 %*]
	rem Load site information
	call :LOADREG
	call :LOADSITE %SITENAME%

	rem Process all command switches
	for /l %%I in (1,1,%CMDSWCOUNT%) do call :REPLEDIT1 %%I

	rem Save edited site information
	call :SAVESITE %SITENAME%
	call :SAVEREG
goto :EOF
:REPLEDIT1
	call _mtplib :GETSWITCH %1
	if "%RET%"=="/add" (
		(call :REPLEDITADD)
	) else if "%RET%"=="/delete" (
		(call :REPLEDITDELETE)
	) else if "%RET%"=="/delall" (
		(call :REPLEDITDELALL)
	) else if "%RET%"=="/enable" (
		(set WORKENABLE=1)
	) else if "%RET%"=="/disable" (
		(set WORKENABLE=0)
	)
goto :EOF

:REPLEDITADD
	if "%RETV%"=="" goto :EOF
	call :REPLEDITDELETE
	set /a WORKCOUNT+=1
	set WORKDIR_%WORKCOUNT%_=%RETV%
goto :EOF

:REPLEDITDELETE
	if "%RETV%"=="" goto :EOF
	set /a IX=1
	:REPLEDITDELETELOOP
		if %IX% GTR %WORKCOUNT% goto :EOF
		set RET=%%WORKDIR_%IX%_%%
		call _mtplib :RESOLVE
		if /i "%RET%"=="%RETV%" set WORKDIR_%IX%_=
		set /a IX+=1
	goto :REPLEDITDELETELOOP
goto :EOF

:REPLEDITDELALL
	if not "%NOCONFIRM%"=="1" (
		%COMSPEC% /c choice /n "Delete all site directories?"
		if errorlevel 2 goto :EOF
	)
	set /a WORKENABLE=0
	set /a WORKCOUNT=0
	call _mtplib :VARDEL WORKDIR_
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem GETREPLJOBID procedure
rem
rem Returns:	RET=job ID of REPL command, or "NONE" if not found
rem
:GETREPLJOBID
if defined TRACE %TRACE% [proc %0 %*]
	set RET=NONE
	for /f "tokens=1" %%I in ('at ^| find "%SCRIPTNAME%"') do set RET=%%I
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem LOADREG procedure
rem Load registry information into variables
rem
:LOADREG
if defined TRACE %TRACE% [proc %0 %*]
	rem Delete existing site information
	call _mtplib :VARDEL RSENABLE_
	call _mtplib :VARDEL RSDIR_

	rem Load site data from the registry
	call _mtplib :REGGETM %CONTEXT%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem SAVEREG procedure
rem Save registry information from variables
rem
:SAVEREG
if defined TRACE %TRACE% [proc %0 %*]
	rem Save site data to the registry
	call _mtplib :REGDELM %CONTEXT%
	call _mtplib :REGSETM %CONTEXT% RSENABLE_
	call _mtplib :REGSETM %CONTEXT% RSDIR_
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem LOADSITE procedure
rem Load site into working vars
rem
rem Arguments:	%1=site name
rem
rem Returns:	WORKENABLE=0/1 for site enable
rem		WORKDIR_n_=list of directories (1 based)
rem		WORKCOUNT=count of directories in list
rem
:LOADSITE
if defined TRACE %TRACE% [proc %0 %*]
	rem Clear existing working site
	set /a WORKENABLE=1
	set /a WORKCOUNT=0
	call _mtplib :VARDEL WORKDIR_
	if not defined RSENABLE_%1 goto :EOF

	rem Load site into working site
	set /a WORKENABLE=RSENABLE_%1
	for /f "tokens=1 delims==" %%I in ('set RSDIR_%1_ 2^>nul') do call :LOADSITE1 %1 %%I
goto :EOF
:LOADSITE1
	for /f "tokens=3 delims=_" %%J in ("%2") do set IX=%%J
	set RET=%%%2%%
	call _mtplib :RESOLVE
	set WORKDIR_%IX%_=%RET%
	set /a WORKCOUNT+=1
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem SAVESITE procedure
rem Save site from working vars, with site compaction
rem
rem Arguments:	%1=site name
rem
:SAVESITE
if defined TRACE %TRACE% [proc %0 %*]
	rem Clear existing site data
	set RSENABLE_%1=
	call _mtplib :VARDEL RSDIR_%1_
	if not defined WORKENABLE goto :EOF
	if not defined WORKCOUNT goto :EOF
	if %WORKCOUNT% EQU 0 goto :EOF

	rem Save working site
	set /a RSENABLE_%1=WORKENABLE
	set /a NEWCOUNT=0
	for /l %%I in (1,1,%WORKCOUNT%) do call :SAVESITE1 %1 %%I
goto :EOF
:SAVESITE1
	if not defined WORKDIR_%2_ goto :EOF
	set RET=%%WORKDIR_%2_%%
	call _mtplib :RESOLVE
	set /a NEWCOUNT+=1
	set RSDIR_%1_%NEWCOUNT%_=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem Additional procedures go here...

rem These must be the FINAL LINES in the script...
:DOSEXIT
echo This script requires Windows NT

rem /////////////////////////////////////////////////////////////////////////
